#include <fcntl.h>
#include "libgraph.h"
#include <alloc.h>
#include <fail.h>
#include <string.h>




value gr_add_window(x, y, w, h, str)	/* ML */
     value x, y, w, h, str;
{
  XSizeHints hints;
  int ret,i;
  XEvent event;
  XWindowAttributes attributes;
  int wx,wy, wh,ww;
  canvas gwindow;
  canvas *tmp;
  unsigned long valuemask;
  XSetWindowAttributes win_att;

  if (! gr_initialized) 
  {
    gr_fail("add_window: graphic mode not initialise\n", "");
    return Val_unit;
  }


#ifdef DEBUG
   fprintf(stderr,"gr_add_window: begin\n");
#endif

    /* Parse the geometry specification */
    hints.x = Int_val(x);
    hints.height = Int_val(h);
    hints.y = Scvt(Int_val(y))-hints.height+1;
    hints.width = Int_val(w);;
    hints.flags = PPosition | PSize;
    hints.win_gravity = 0;

#ifdef DEBUG
   fprintf(stderr,"gr_add_window: x=%d, y=%d\n",hints.x, hints.y);
   fprintf(stderr,"gr_add_window: w=%d, h=%d\n",hints.width, hints.height);
#endif

    ret = XWMGeometry(grdisplay, grscreen, "", "", BORDER_WIDTH,
                      &hints, &wx, &wy, &ww, &wh, &hints.win_gravity);
    if (ret & (XValue | YValue)) {
      hints.x = wx; hints.y = wy; hints.flags |= USPosition;
    }
    if (ret & (WidthValue | HeightValue)) {
      hints.width = ww; hints.height = wh; hints.flags |= USSize;
    }

 
    /* Create the on-screen window */
    gwindow.w = hints.width;
    gwindow.h = hints.height;
    if(hints.x<0) hints.x =0;
    if(hints.y+hints.height>grscreen_height) hints.y = grscreen_height - hints.height-10;

    gwindow.win =
      XCreateSimpleWindow(grdisplay, DefaultRootWindow(grdisplay),
                          hints.x, hints.y, hints.width, hints.height,
                          BORDER_WIDTH, grblack, grwhite);
    XSetStandardProperties(grdisplay, gwindow.win, String_val(str), ICON_NAME,
                           None, NULL, 0, &hints);
    gwindow.gc = XCreateGC(grdisplay, gwindow.win, 0, NULL);
    XSetBackground(grdisplay, gwindow.gc, grwhite);
    XSetForeground(grdisplay, gwindow.gc, grcolor);


  if(DoesBackingStore(DefaultScreenOfDisplay(grdisplay))!=NotUseful)
    {
      valuemask = CWBackingStore;
      win_att.backing_store= WhenMapped; 
      XChangeWindowAttributes(grdisplay, gwindow.win, valuemask, &win_att);
    }

  /* Position the current point at origin */
  gwindow.grx = 0;
  gwindow.gry = 0;

  gwindow.grfont = NULL;
  gwindow.underline = FALSE;

  /* add the window in the window list */
  tmp=(canvas *)stat_alloc((nb_windows+1)*sizeof(canvas));
  if(tmp==NULL)
  {
    gr_fail("%s","gr_add_window : not enougth memory");
  }
  for(i=0;i<nb_windows;i++)
    *(tmp+i)=*(grwindows+i);
  tmp[nb_windows]=gwindow;
  stat_free(grwindows);
  grwindows=tmp;
  nb_windows++;
  current_window=nb_windows-1;


 
   
    /* Require exposure, resize and keyboard events */
    XSelectInput(grdisplay, gwindow.win, DEFAULT_EVENT_MASK);

    /* Map the window on the screen and wait for the first Expose event */
    XMapWindow(grdisplay, gwindow.win);
    XFlush(grdisplay);

  ValidWindow=TRUE;
#ifdef DEBUG
   fprintf(stderr,"gr_add_window: call gr_set_font\n");
#endif
 gr_set_font(Atom(0), Atom(0), Atom (0));

  #ifdef DEBUG
     fprintf(stderr,"gr_add_window: return id %d\n",gwindow.win);
  #endif

  
  return Val_int(gwindow.win);
}

value gr_del_window(win_id)  /* ML */
    value win_id;
{
  int i,id=Int_val(win_id),j;
  canvas *tmp;
  
  #ifdef DEBUG
     fprintf(stderr,"gr_del_window : window id=%d\n",id);
  #endif

  for (i=0; i<nb_windows; i++)
    if(grwindows[i].win==id) break;

  if(i==nb_windows) 
  {
    #ifdef DEBUG
       fprintf(stderr,"gr_del_window: not valid id, end\n");
    #endif

    return Val_unit;
  }

  if (grwindows[i].grfont != NULL) 
  { 
    XFreeFont(grdisplay, grwindows[i].grfont); 
    grwindows[i].grfont = NULL; 
  }
  XFreeGC(grdisplay, grwindows[i].gc);
  XDestroyWindow(grdisplay, grwindows[i].win);

     /* del the window in the window list */
  tmp=(canvas *)stat_alloc((nb_windows-1)*sizeof(canvas));
  if(tmp==NULL)
  {
     gr_fail("%s","gr_del_window : not enougth memory");
  }
  for(j=0;j<i;j++)
    *(tmp+j)=*(grwindows+j);
  for(;j<nb_windows-1;j++)
    *(tmp+j)=*(grwindows+j+1);
  stat_free(grwindows);
  grwindows=tmp;
  nb_windows--;
  if(current_window==i) current_window=nb_windows-1;
  ValidWindow=TRUE;

  #ifdef DEBUG
     fprintf(stderr,"gr_del_window: valid id, end\n");
  #endif


  return Val_unit;
}


value gr_set_draw_window(id_win) /* ML */
  value id_win;
{
  int i,id=Int_val(id_win);;

#ifdef DEBUG
  fprintf(stderr,"gr_set_draw_window: begin. ",id);
#endif

  if(id==grwindows[current_window].win) 
  {
    #ifdef DEBUG
       fprintf(stderr,"current window\n",id);
    #endif

    ValidWindow=TRUE;
    return Val_unit;
  }

  for (i=0; i<nb_windows; i++)
    if(grwindows[i].win==id) break;

  if(i==nb_windows)
  {
    ValidWindow=FALSE;
    current_window=0;

    #ifdef DEBUG
       fprintf(stderr,"id=%d not valid\n",id);
    #endif

    return Val_unit;
  }

  #ifdef DEBUG
     fprintf(stderr,"id=%d valid\n",id);
  #endif

  ValidWindow=TRUE;
  current_window=i;
  return Val_unit;
}

value gr_set_current_window(id_win) /* ML */
  value id_win;
{
  int i,id=Int_val(id_win);

  if(id==grwindows[current_window].win) return Val_unit;

  for (i=0; i<nb_windows; i++)
    if(grwindows[i].win==id) break;

  if(i==nb_windows)
  {
    ValidWindow=FALSE;
    current_window=0;

    #ifdef DEBUG
       fprintf(stderr,"gr_set_current_window: id=%d not valid\n",id);
    #endif

    return Val_unit;
  }

  #ifdef DEBUG
     fprintf(stderr,"gr_set_current_window: id=%d valid\n",id);
  #endif
  
  ValidWindow=TRUE;
  current_window=i;
  XRaiseWindow(grdisplay,grwindows[i].win);
  return Val_unit;
}

value gr_get_current_window()    /* ML */
{
  int id=grwindows[current_window].win;

  #ifdef DEBUG
     fprintf(stderr,"gr_get_current_window: retrun %d\n",id);
  #endif

  return Val_int(id);
}

value gr_window_pos(id_win)     /* ML */
  value id_win;
{ 
  Window root;
  Window unused;
  int x, y, ab_x, ab_y;
  unsigned int width,height;
  unsigned int border_w;
  unsigned int depth;  
  int i,id=Int_val(id_win);
  value res;

  #ifdef DEBUG
     fprintf(stderr,"gr_window_pos: id=%d\n",id);
  #endif


  for (i=0; i<nb_windows; i++)
    if(grwindows[i].win==id) break;

  if(i==nb_windows)
  {    
    #ifdef DEBUG
       fprintf(stderr,"gr_window_pos: id not valid, retrun (0,0)\n");
    #endif

    res = alloc_tuple(2);
    Field(res, 0) = Val_int(0);
    Field(res, 1) = Val_int(0);
    return res;
  }
  
  XGetGeometry(grdisplay,grwindows[i].win, &root, &x, &y,
               &width, &height, &border_w, &depth);

  XTranslateCoordinates(grdisplay, grwindows[i].win, root, x, y,
                       &ab_x, &ab_y, &unused);
 
  #ifdef DEBUG
     fprintf(stderr,"gr_window_pos: id valid, retrun (%d,%d)\n",
             ab_x, ab_y-height);
  #endif

  res = alloc_tuple(2);
  Field(res, 0) = Val_int(ab_x);
  Field(res, 1) = Val_int(grscreen_height-ab_y-height);
  return res;
}


value gr_window_width(id_win)	/* ML */
  value id_win;
{
  Window root;
  int x,y;
  unsigned int width,height;
  unsigned int border_w;
  unsigned int depth;  
  int i,id=Int_val(id_win);

  #ifdef DEBUG
     fprintf(stderr,"gr_window_width: id=%d\n",id);
  #endif

  for (i=0; i<nb_windows; i++)
    if(grwindows[i].win==id) break;

  if(i==nb_windows)
  {
    #ifdef DEBUG
       fprintf(stderr,"gr_window_width: not valid id, retrun 0\n");
    #endif

    return Val_int(0);
  }
  
  XGetGeometry(grdisplay,grwindows[i].win,&root,&x,&y,
               &width,&height,&border_w,&depth);
  #ifdef DEBUG
     fprintf(stderr,"gr_window_width: id valid, retrun %d\n",width);
  #endif

  return Val_int(width);
}

value gr_window_height(id_win)	/* ML */
  value id_win;
{
  Window root;
  int x,y;
  unsigned int width,height;
  unsigned int border_w;
  unsigned int depth;
  int i,id=Int_val(id_win);

  for (i=0; i<nb_windows; i++)
    if(grwindows[i].win==id) break;

  if(i==nb_windows)
  {
   #ifdef DEBUG
       fprintf(stderr,"gr_window_height: id not valid, retrun 0\n");
    #endif

    return Val_int(0);
  }

  XGetGeometry(grdisplay,grwindows[i].win,&root,&x,&y,
               &width,&height,&border_w,&depth);
  #ifdef DEBUG
     fprintf(stderr,"gr_window_height: id valid, retrun %d\n",height);
  #endif

  return Val_int(height);  
}

value gr_set_window_car(x, y, w, h)         /* ML */
  value x, y, w, h;
{
  int xw,yw,ww,hw;
  
  xw=Int_val(x);
  yw=Int_val(y);
  ww=Int_val(w);
  hw=Int_val(h);

  if(ValidWindow==FALSE) return Val_unit;
  #ifdef DEBUG
     fprintf(stderr,"gr_set_window_car: end\n");
  #endif

  XMoveResizeWindow(grdisplay,grwindows[current_window].win,
                    xw,Scvt(yw+hw),ww,hw);
  return Val_unit;
}



value gr_default_window ()
{    
#ifdef DEBUG
       fprintf(stderr,"gr_default_window\n");
#endif
  return Val_int(0);
}




